﻿/*
  In App.xaml:
  <Application.Resources>
      <vm:InjectedViewModelLocator xmlns:vm="clr-namespace:WMAApp.Locators"
                                   x:Key="Locator" />
  </Application.Resources>
  
  In the View:
  DataContext="{Binding Source={StaticResource Locator}, Path=ViewModelName}"
 
  Add references to: System.ComponentModel.Composition
                     System.ComponentModel.Composition.Initialization
*/
#region Using
using System;
using System.Linq;
using System.Diagnostics;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.ComponentModel.Composition;
using System.ComponentModel.Composition.Hosting;

// Toolkit namespace
using SimpleMvvmToolkit;
using WMAApp.Services;
using WMAApp.ViewModels.Home;
using WMAApp.ViewModels.Main;
using WMAApp.Services.Connection;
using WMAApp.Services.Home;
using WMAApp.ViewModels.Connection;
#endregion Using

namespace WMAApp
{
    /// <summary>
    /// <para>
    /// This class creates ViewModels on demand for Views, supplying a
    /// ServiceAgent to the ViewModel if required.
    /// It uses Managed Extensibility Framework (MEF) to inject service agents,
    /// which need to decorate themselves with this attribute:
    /// [ServiceAgentExport(typeof(IXXXServiceAgent), AgentType = AgentType.Real)]
    /// AgentType can be set to Real or Mock.
    /// Place the ViewModelLocator in the App.xaml resources:
    /// </para>
    /// <code>
    /// &lt;Application.Resources&gt;
    ///     &lt;vm:InjectedViewModelLocator xmlns:vm="clr-namespace:WMAApp.Locators"
    ///                                  x:Key="Locator" /&gt;
    /// &lt;/Application.Resources&gt;
    /// </code>
    /// <para>
    /// Then use:
    /// </para>
    /// <code>
    /// DataContext="{Binding Source={StaticResource Locator}, Path=ViewModelName}"
    /// </code>
    /// <para>
    /// Use the <strong>mvvminjectedlocator</strong> or <strong>mvvmlocatornosa</strong>
    /// code snippets to add ViewModels to this locator.
    /// </para>
    /// </summary>
    public class InjectedViewModelLocator
    {
        static InjectedViewModelLocator()
        {
            // Expose parts in the current XAP to MEF
            if (!DesignerProperties.IsInDesignTool)
                CompositionHost.Initialize(new DeploymentCatalog());
        }

        // default service agent type
        private AgentType agentType = AgentType.Real;
        public AgentType AgentType
        {
            get { return agentType; }
            set { agentType = value; }
        }

        /// <summary>
        /// MEF will create service agents based on ServiceAgentExport attribute.
        /// There is a reference to InjectedViewModelLocator on App.xaml that causes contructor to be invoked.
        /// </summary>
        public InjectedViewModelLocator()
        {
            // Only inject service agents at runtime
            if (!DesignerProperties.IsInDesignTool)
            {
                // Use MEF to inject service agents
                CompositionInitializer.SatisfyImports(this);

                //( I don't think that these are intended to be the only objects of their type. I
                // think that this approach requires new viewmodels to be created on the fly when the views that house them are
                // instantiated/shown. This code just verifies that they are creatable. )

                // Verify service agent creation.
                //Debug.Assert(this.DbSelectVM != null,
                //    string.Format("DbSelectVM could not be created."));
                //Debug.Assert(this.HomeVM != null,
                //    string.Format("No HomeSA has ServiceAgentExport with AgentType = {0}", agentType));                
            }
        }

        #region view models

        // TODO: Use mvvminjectedlocator or mvvmlocatornosa code snippets to add ViewModels to the locator.


        // Create MainPageViewModel on demand
        public MainPageVM MainPageVM
        {
            get { return new MainPageVM(); }
        }

        // NOTE: Service agents are only used with viewmodels that can use mock, real db connections etc.
        // Create HomeVM on demand
        [ImportMany] 
        public Lazy<IHomeSA, IServiceAgentMetadata>[] HomeSAs { get; set; }
        public HomeVM HomeVM
        {
            get
            {
                var serviceAgent = HomeSAs
                    .Where(sa => sa.Metadata.AgentType == agentType).FirstOrDefault();
                HomeVM viewModel = null;
                if (serviceAgent != null)
                    viewModel = new HomeVM(serviceAgent.Value);
                else if (DesignerProperties.IsInDesignTool)
                    viewModel = new HomeVM();
                return viewModel;
            }
        }

        //Create DbSelectVM on demand
        
        //[ImportMany] // This approach not needed since DbSelectVM doesn't need to access a db through a service agent?
        //public Lazy<IDbSelectSA, IServiceAgentMetadata>[] DbSelectSAs { get; set; }
        //public DbSelectVM DbSelectVM
        //{
        //    get
        //    {
        //        var serviceAgent = DbSelectSAs
        //            .Where(sa => sa.Metadata.AgentType == agentType).FirstOrDefault();
        //        DbSelectVM viewModel = null;
        //        if (serviceAgent != null)
        //            viewModel = new DbSelectVM(serviceAgent.Value);
        //        else if (DesignerProperties.IsInDesignTool)
        //            viewModel = new DbSelectVM();
        //        return viewModel;
        //    }
        //}
        public DbSelectVM DbSelectVM
        {
            get { return new DbSelectVM(); }
        }

        // Create ConnectStatusVM on demand
        public ConnectStatusVM ConnectStatusVM
        {
            get { return new ConnectStatusVM(); }
        }

        //TODO: create additional views models here.

        // Example: Create ItemListViewModel on demand
        //[ImportMany]
        //public Lazy<IItemListServiceAgent, IServiceAgentMetadata>[] ItemListServiceAgents { get; set; }
        //public ItemListViewModel ItemListViewModel
        //{
        //    get
        //    {
        //        var serviceAgent = ItemListServiceAgents
        //            .Where(sa => sa.Metadata.AgentType == agentType).FirstOrDefault();
        //        ItemListViewModel viewModel = null;
        //        if (serviceAgent != null) viewModel = new ItemListViewModel(serviceAgent.Value);
        //        else if (DesignerProperties.IsInDesignTool) viewModel = new ItemListViewModel();
        //        return viewModel;
        //    }
        //}
        #endregion

    }
}